/*
 * @Description: 包装明细相关
 * @Author: Ben Lin
 * @version:
 * @Date: 2024-06-19 20:34:27
 * @LastEditors: Ben Lin
 * @LastEditTime: 2024-09-11 20:51:11
 */

import { Ref, ref, unref } from 'vue';
import { DeleteEntity, getEntity, SaveEntity } from '/@/api/tigerapi/system';
import { ActionItem, BasicColumn, FormSchema, useTable } from '/@/components/Table';
import { isNullOrEmpty, isNullOrUnDef } from '/@/utils/is';
import { useDrawer } from '/@/components/Drawer';
import { EditOperation, Search, custDel } from '../data';
import { SaveBasPkgRule } from '/@/api/tigerapi/mes/baserule';
import { BAS_PKG_RULE } from '/@/api/tigerapi/model/mesModel';

function _default(): any[] {
  const ActionColumn: BasicColumn = {
    width: 80,
    title: '操作',
    dataIndex: 'action',
    slots: { customRender: 'action' },
    fixed: undefined,
  };

  let _data = ref<any>({});
  let record = {};
  /**
   * @description: 一些自定义方法
   * @return {*}
   */
  const methods = {
    /**
     * @description: 获取新增按钮的行为
     * @return {*}
     */
    CreateAction: (type: string) => {
      return {
        action: 'drawer', //drawer(打开左侧抽屉框) | go(跳转到新的页面)
      };
    },
    /**
     * @description: 操作字段自定义按钮
     * @return {*}
     */
    ActionItem: (params: Recordable<any>, data, ...args): ActionItem[] => {
      /* 重写删除方法 */
      if (!isNullOrUnDef(data[1])) {
        data[1].popConfirm.confirm = Del.bind(null, args, params);
      }
      return data;
    },
    /**
     * @description: 自定义明细表中编辑返回方法
     * @param {string} type
     * @param {*} d
     * @param {*} u
     * @return {*}
     */
    EditOperation: (data: Ref<any[]>, d, u, name) => {
      //更新
      var _data = data.value[name].map((item) => {
        if (item['ID'] == d.ID) {
          if (name == 'BAS_PKG_DTL') {
            return {
              ...item,
              PKG_CODE: d.PKG_CODE,
              PKG_NAME: d.PKG_NAME,
              PKG_LEVEL: d.PKG_LEVEL,
              PKG_QTY: d.PKG_QTY,
              LABEL_CODE: d.LABEL_CODE,
              REMARK: d.REMARK,
            };
          }
          if (name == 'BAS_PKG_PROD') {
            return {
              ...item,
              ITEM_CODE: d.ITEM_CODE,
              MIN_WEIGHT: d.MIN_WEIGHT,
              MIN_UNIT: d.MIN_UNIT,
              MAX_WEIGHT: d.MAX_WEIGHT,
              MAX_UNIT: d.MAX_UNIT,
              REMARK: d.REMARK,
            };
          }
        }
        return item;
      });
      EditOperation(data, d, u, name, _data);
    },
    /**
     * @description: 高级表单和详情页面返回主页面的url
     * @return {*}
     */
    GetHomeUrl: () => {
      return `/BAS_PKG_RULE/LC/${encodeURI(JSON.stringify({ ID: 'BAS_PKG_RULE', colSlots: [], crudColSlots: [] }))}`;
    },
    /**
     * @description: 高级表单明细表格字段
     * @return {*}
     */
    GetBaseColumns: (type: string) => {
      return baseColumns[type];
    },
    /**
     * @description: 高级表单查询条件字段
     * @return {*}
     */
    GetSearchForm: (type: string) => {
      return searchForms[type];
    },
    /**
     * @description: 高级表单增删改表单字段
     * @return {*}
     */
    GetCrudForm: (type: string, ...args) => {
      _data = args[0];
      record = args[1];
      return crudForms[type];
    },
    KeyFieldValues: (val: string, id: string) => {
      return { PKG_RULE_ID: id };
    },
    /**
     * @description: 高级表单主基本信息字段
     * @return {*}
     */
    GetBaseForm: () => {
      return [
        {
          field: 'RULE_CODE',
          label: '包装规则编码',
          component: 'Input',
          required: true,
          colProps: {
            span: 8,
          },
        },
        {
          label: '包装规则名称',
          field: 'RULE_NAME',
          required: true,
          component: 'Input',
          colProps: {
            span: 8,
          },
        },
        {
          label: '是否启用',
          field: 'IS_ACTIVE',
          required: true,
          component: 'Select',
          colProps: {
            span: 8,
          },
          componentProps: {
            options: [
              {
                label: '启用',
                value: 'Y',
                key: 'Y',
              },
              {
                label: '禁用',
                value: 'N',
                key: 'N',
              },
            ],
          },
        },
        {
          label: '备注',
          field: 'REMARK',
          component: 'Input',
          colProps: {
            span: 8,
          },
        },
        {
          label: 'ID',
          field: 'ID',
          component: 'Input',
          colProps: {
            span: 8,
          },
          show: false,
        },
      ];
    },
    /**
     * @description: 高级表单主信息卡片生成，可以多个
     * @return {*}
     */
    GetBaseCards: () => {
      return [
        {
          name: 'BaseForm',
          slots: [], //定义插槽
          preIcons: {}, //定义插槽中按钮图标
          title: '包装信息',
          entityName: 'BAS_PKG_RULE', //实体名称
        },
      ];
    },
    /**
     * @description: 获取右侧边框use方法
     * @return {*}
     */
    GetUseDrawers: () => {
      return [
        {
          BAS_PKG_DTL: useDrawer(),
        },
        {
          BAS_PKG_PROD: useDrawer(),
        },
      ];
    },
    /**
     * @description: 获取表格use列表
     * @param {string} type
     * @param {array} args
     * @return {*}
     */
    GetUseTables: (data: Ref<{}>, ...args) => {
      return {
        BAS_PKG_DTL: useTable({
          title: '列表信息',
          dataSource: data.value['BAS_PKG_DTL'],
          columns: baseColumns['BAS_PKG_DTL'],
          formConfig: {
            labelWidth: 140,
            schemas: searchForms['BAS_PKG_DTL'],
            submitFunc: () => Search('BAS_PKG_DTL', data, args[0]), //自定义查询提交按钮的方法，触发查询提交事件
          },
          useSearchForm: true,
          showTableSetting: false,
          bordered: true,
          canResize: true,
          showIndexColumn: false,
          actionColumn: {
            width: 130,
            title: '操作',
            dataIndex: 'action',
            slots: { customRender: 'action' },
            fixed: 'right',
          }, //自定义操作列
        }),
        BAS_PKG_PROD: useTable({
          title: '列表信息',
          dataSource: data.value['BAS_PKG_PROD'],
          columns: baseColumns['BAS_PKG_PROD'],
          formConfig: {
            labelWidth: 140,
            schemas: searchForms['BAS_PKG_PROD'],
            submitFunc: () => Search('BAS_PKG_PROD', data, args[0]), //自定义查询提交按钮的方法，触发查询提交事件
          },
          useSearchForm: true,
          showTableSetting: false,
          bordered: true,
          canResize: true,
          showIndexColumn: false,
          actionColumn: {
            width: 130,
            title: '操作',
            dataIndex: 'action',
            slots: { customRender: 'action' },
            fixed: 'right',
          }, //自定义操作列
        }),
      };
    },
    /**
     * @description: 表格查询回调，父组件中通过子组件触发的事件获取传入的值，做具体的查询逻辑实现
     * @param {*} d
     * @return {*}
     */
    FormSearch: (d) => {
      let data = {} as any;
      switch (d.type) {
        case 'BAS_PKG_DTL':
          data = d.data.value[d.type].filter((item) => item.PKG_CODE.includes(d.values.PKG_CODE));
          if (isNullOrEmpty(d.values.PKG_CODE)) {
            data = d.data.value[d.type];
          }
          break;
        case 'BAS_PKG_PROD':
          data = d.data.value[d.type].filter((item) => item.ITEM_CODE.includes(d.values.ITEM_CODE));
          if (isNullOrEmpty(d.values.ITEM_CODE)) {
            data = d.data.value[d.type];
          }
          break;
      }
      return data;
    },
    /**
     * @description: 获取标题信息
     * @param {string} type
     * @return {*}
     */
    GetTitle: () => {
      return {
        pageTitle: '包装规则管理',
        pageContent: '这里可以添加和修改包装规则和包装信息。',
        tableTitle: {
          BAS_PKG_DTL: '包装信息管理',
          BAS_PKG_PROD: '称重范围管理',
        },
      };
    },
    GetUseForm: () => {
      return {};
    },
    /**
     * @description: 提交所有
     * @return {*}
     */
    SubmitAll: (data: Ref<any[]>, keyFieldValues: Ref<{}>, ...args) => {
      const validates = args[1].validates;
      const Keys = args[1].Keys;
      const isAllUpdate = args[1].isAllUpdate;
      const baseCards = args[1].baseCards;
      const IsExist = args[1].IsExist;
      /* 循环保存高级表单中主表单信息，如果有多个的话，所以要循环 */
      let i;
      let p = [] as Promise<any>[];
      for (i = 0; i < Keys.length; i++) {
        p.push(
          SaveEntity(
            validates[Keys[i]],
            unref(isAllUpdate),
            baseCards.value[i]['entityName'],
            `${IsExist}='${validates[Keys[i]][IsExist]}'`,
            true,
          ),
        );
      }
      Promise.all(p).then(() => {
        SaveBasPkgRule({
          PkgDtl: data.value['BAS_PKG_DTL'],
          PkgProd: data.value['BAS_PKG_PROD'],
          PkgRule: {} as unknown as BAS_PKG_RULE,
        }).then((action) => {
          if (action.IsSuccessed) {
            args[0]();
          }
        });
      });
    },
  };

  /* 以下是内部方法，不export，供上面的方法调用 */

  /**
   * @description: 自定义删除方法
   * @param {Fn} args
   * @param {*} params
   * @return {*}
   */
  function Del(args: Fn[], params: {}) {
    const name = params['name'];
    const useTables = args[1];
    if (!isNullOrEmpty(params['data'])) {
      var _data = params['data'].value[name]
        .filter((x) => x['ID'] != params['record']['ID']) //过滤掉删除掉的记录
        .map((item) => {
          //如果序号大于删除掉的记录的序号的，则序号减1
          item['PKG_LEVEL'] =
            item['PKG_LEVEL'] > params['record']['PKG_LEVEL']
              ? item['PKG_LEVEL'] - 1
              : item['PKG_LEVEL'];
          return item;
        });
      params['data'].value[name] = _data;
      useTables[name][1].setProps({
        dataSource: [],
      });
      useTables[name][1].setProps({
        dataSource: params['data'].value[name],
      });
      useTables[name][1].reload();
    } else {
      DeleteEntity(params['record'], params['entityName']).then((action) => {
        if (action.IsSuccessed) {
          useTables[name][1].reload();
        }
      });
    }
  }

  const searchForms = {
    BAS_PKG_DTL: [
      {
        field: 'PKG_CODE',
        label: '包装规则编码',
        component: 'Input',
        colProps: {
          span: 8,
        },
      },
    ] as FormSchema[],
    BAS_PKG_PROD: [
      {
        field: 'ITEM_CODE',
        label: '物料编码',
        component: 'Input',
        colProps: {
          span: 8,
        },
      },
    ] as FormSchema[],
  };

  const baseColumns = {
    BAS_PKG_DTL: [
      {
        title: '包装/载具编码',
        dataIndex: 'PKG_CODE',
        ifShow: false,
        width: 180,
      },
      {
        title: '包装/载具名称',
        dataIndex: 'PKG_NAME',
        ifShow: false,
        width: 180,
      },
      {
        title: '包装/载具',
        dataIndex: 'PKG_FULL',
        width: 180,
        customRender: ({ record }) => {
          return `${record.PKG_CODE} - ${record.PKG_NAME}`;
        },
      },
      {
        title: '包装层级',
        dataIndex: 'PKG_LEVEL',
      },
      {
        title: '包装数量',
        dataIndex: 'PKG_QTY',
      },
      {
        title: '标签模板',
        dataIndex: 'LABEL_CODE',
      },
      {
        title: '包装规则ID',
        dataIndex: 'PKG_RULE_ID',
        ifShow: false,
      },
      {
        title: '备注',
        dataIndex: 'REMARK',
      },
      {
        title: '更新时间',
        dataIndex: 'UPDATE_TIME',
      },
      {
        title: '更新人',
        dataIndex: 'UPDATE_USER',
      },
    ] as BasicColumn[],
    BAS_PKG_PROD: [
      {
        title: '物料编码',
        dataIndex: 'ITEM_CODE',
      },
      {
        title: '称重下限',
        dataIndex: 'MIN_WEIGHT',
      },
      {
        title: '称重下限单位',
        dataIndex: 'MIN_UNIT',
      },
      {
        title: '称重上限',
        dataIndex: 'MAX_WEIGHT',
      },
      {
        title: '称重上限单位',
        dataIndex: 'MAX_UNIT',
      },
      {
        title: '包装规则ID',
        dataIndex: 'PKG_RULE_ID',
        ifShow: false,
      },
      {
        title: '备注',
        dataIndex: 'REMARK',
      },
      {
        title: '更新时间',
        dataIndex: 'UPDATE_TIME',
      },
      {
        title: '更新人',
        dataIndex: 'UPDATE_USER',
      },
    ] as BasicColumn[],
  };

  const crudForms = {
    BAS_PKG_DTL: [
      {
        label: '包装规则ID',
        field: 'PKG_RULE_ID',
        component: 'Input',
        colProps: {
          span: 24,
        },
        dynamicDisabled: ({ values }) => {
          return true;
        },
      },
      {
        label: '包装层级',
        field: 'PKG_LEVEL',
        component: 'Input',
        required: true,
        colProps: {
          span: 24,
        },
        componentProps: ({ formModel }) => {
          if (isNullOrEmpty(formModel.PKG_LEVEL) || Number(formModel.PKG_LEVEL) == 0) {
            formModel.PKG_LEVEL =
              _data['BAS_PKG_DTL'].reduce((max, item) => {
                return max < Number(item.PKG_LEVEL) ? Number(item.PKG_LEVEL) : max;
              }, 0) + 1;
          }
          return {
            onChange: (e) => {},
          };
        },
        dynamicDisabled: ({ values }) => {
          return true;
        },
      },
      {
        field: 'PKG_CODE',
        label: '包装/载具编码',
        component: 'ApiSelect',
        required: true,
        colProps: {
          span: 24,
        },
        componentProps: ({ formModel }) => {
          let sqlcmd = ' 1=1 ';
          if (!isNullOrEmpty(record) && record['PKG_LEVEL'] == 1) {
            sqlcmd += ` And IS_MIN_PKG = 'Y'`;
          } else {
            if (formModel.PKG_LEVEL == 1) {
              sqlcmd += ` And IS_MIN_PKG = 'Y'`;
            } else {
              sqlcmd += ` And IS_MIN_PKG != 'Y'`;
            }
          }
          return {
            api: getEntity,
            params: { entityName: 'BAS_PKG_TYPE', sqlcmd: sqlcmd },
            resultField: 'Data.Items',
            labelField: 'PKG_NAME',
            valueField: 'PKG_CODE',
            onChange: (e, v) => {
              console.log('ApiSelect====>:', e, v);
              formModel.PKG_NAME = isNullOrUnDef(v) ? formModel.PKG_NAME : v.label;
            },
          };
        },
      },
      {
        label: '包装数量',
        field: 'PKG_QTY',
        component: 'Input',
        colProps: {
          span: 24,
        },
      },
      {
        field: 'LABEL_CODE',
        label: '模板编码',
        component: 'Input',
        show: true,
        // required: true,
        colProps: {
          span: 22,
        },
      },
      {
        field: 'BAS_LABEL_TEMP1PSelect_0', //按低代码配置的规则，实体名+序号+PSelect_0，序号用来区分多个的时候，PSelect_0这是个固定后缀
        label: '1',
        defaultValue: 'BAS_LABEL_TEMP',
        component: 'Input',
        colProps: { span: 2 },
        colSlot: 'BAS_LABEL_TEMP1add', //按低代码配置的规则，实体名+序号+add，序号用来区分多个的时候，add这是个固定后缀
      },
      {
        label: '包装名称',
        field: 'PKG_NAME',
        component: 'Input',
        colProps: {
          span: 24,
        },
        show: false,
      },
      {
        label: '备注',
        field: 'REMARK',
        component: 'Input',
        colProps: {
          span: 24,
        },
      },
      {
        label: 'ID',
        field: 'ID',
        component: 'Input',
        colProps: {
          span: 24,
        },
        show: false,
      },
    ] as FormSchema[],
    BAS_PKG_PROD: [
      {
        label: '包装规则ID',
        field: 'PKG_RULE_ID',
        component: 'Input',
        colProps: {
          span: 24,
        },
        dynamicDisabled: ({ values }) => {
          return true;
        },
      },
      {
        label: '物料编码',
        field: 'ITEM_CODE',
        component: 'Input',
        required: true,
        colProps: {
          span: 22,
        },
      },
      {
        field: 'BAS_ITEM1PSelect_0', //按低代码配置的规则，实体名+序号+PSelect_0，序号用来区分多个的时候，PSelect_0这是个固定后缀
        label: '1',
        defaultValue: 'BAS_ITEM',
        component: 'Input',
        colProps: { span: 2 },
        colSlot: 'BAS_ITEM1add', //按低代码配置的规则，实体名+序号+add，序号用来区分多个的时候，add这是个固定后缀
      },
      {
        label: '称重下限',
        field: 'MIN_WEIGHT',
        component: 'Input',
        required: true,
        colProps: {
          span: 24,
        },
      },
      {
        label: '称重下限单位',
        field: 'MIN_UNIT',
        required: true,
        component: 'Input',
        colProps: {
          span: 24,
        },
      },
      {
        label: '称重上限',
        field: 'MAX_WEIGHT',
        required: true,
        component: 'Input',
        colProps: {
          span: 24,
        },
      },
      {
        label: '称重上限单位',
        field: 'MAX_UNIT',
        required: true,
        component: 'Input',
        colProps: {
          span: 24,
        },
      },
      {
        label: '备注',
        field: 'REMARK',
        component: 'Input',
        colProps: {
          span: 24,
        },
      },
      {
        label: 'ID',
        field: 'ID',
        component: 'Input',
        colProps: {
          span: 24,
        },
        show: false,
      },
    ] as FormSchema[],
  };

  return [methods, ActionColumn];
}

export default _default;
